from modelservicecontext import EModelContextStatus
from modelservicecontext import ERequestResponseDataFlag
from modelservicecontext import ERequestResponseDataMIME
from modelservicecontext import ModelServiceContext
from modeldatahandler import ModelDataHandler
import datetime
import sys
import zipfile
import os
import numpy as np
from sklearn.tree import DecisionTreeRegressor  
from sklearn.ensemble import RandomForestRegressor
from sklearn.metrics import r2_score
from sklearn.metrics import mean_squared_error
from math import sqrt
import pickle


if len(sys.argv) < 4:
    exit()

ms = ModelServiceContext()
ms.onInitialize(sys.argv[1], sys.argv[2], sys.argv[3])
mdh = ModelDataHandler(ms)

# State - PM25DataMatching
ms.onEnterState('Predicting')

# Event - load site data
ms.onFireEvent('Model')

ms.onRequestData()

datafile = None
if ms.getRequestDataFlag() == ERequestResponseDataFlag.ERDF_OK:
    if ms.getRequestDataMIME() == ERequestResponseDataMIME.ERDM_RAW_FILE:
        datafile = ms.getRequestDataBody()
    if ms.getRequestDataMIME() == ERequestResponseDataMIME.ERDM_ZIP_FILE:
        datafile = ms.getRequestDataBody()
        ms.mapZipToCurrentDataDirectory(datafile)
        datafile = ms.getDataFileByExt('csv')[0]
else:
    ms.onFinalize()

with open(datafile, 'r') as model_f:
    rf_f = pickle.load(model_f)

# Event - load site data
ms.onFireEvent('PredictionData')

ms.onRequestData()

datafile = None
if ms.getRequestDataFlag() == ERequestResponseDataFlag.ERDF_OK:
    if ms.getRequestDataMIME() == ERequestResponseDataMIME.ERDM_RAW_FILE:
        datafile = ms.getRequestDataBody()
    if ms.getRequestDataMIME() == ERequestResponseDataMIME.ERDM_ZIP_FILE:
        datafile = ms.getRequestDataBody()
        ms.mapZipToCurrentDataDirectory(datafile)
        datafile = ms.getDataFileByExt('csv')[0]
else:
    ms.onFinalize()

predictData = np.loadtxt(datafile)

result = rf_f.predict(predictData)

ms.onLeaveState()

# State - Validation
ms.onEnterState('Validation')

# Event - Load validation real data
ms.onFireEvent('RealData')

datafile = None
if ms.getRequestDataFlag() == ERequestResponseDataFlag.ERDF_OK:
    if ms.getRequestDataMIME() == ERequestResponseDataMIME.ERDM_RAW_FILE:
        datefile = ms.getRequestDataBody()
    if ms.getRequestDataMIME() == ERequestResponseDataMIME.ERDM_ZIP_FILE:
        datefile = ms.getRequestDataBody()
        ms.mapZipToCurrentDataDirectory(datefile)
        datefile = ms.getDataFileByExt('csv')[0]
else:
    ms.onFinalize()

realData = np.loadtxt(datafile)

score_r2 = r2_score(realData, result)
score_rmse = sqrt(mean_squared_error(realData, result))

# Event - Load validation real data
ms.onFireEvent('Result')

resultFile = ms.getCurrentDataDirectory() + "result.csv"
f = open(resultFile, 'w')

count = len(realData)
f.write('Y,Y~\n')
for i in range(count):
    f.write( str(realData[i]) + ',' + str(result[i]) + '\n')

importance = sorted(zip(map(lambda x: round(x, 4), rf_f.feature_importances_), ["landuse", "tmpmax", "tmpmean", "tmpmin", "pcp", "windmean", "windmax", "sd", "rhmin", "rhmean", "prsmean", "prsmin", "prsmax", "evamin", "evamax"]), reverse=True)
f.write('\nFeatrue, Importance\n')
for iItem in importance:
    f.write(iItem[1] + ',' + str(iItem[0]) + '\n')


f.close()

ms.setResponseDataFlag(ERequestResponseDataFlag.ERDF_OK)

ms.setResponseDataMIME(ERequestResponseDataMIME.ERDM_RAW_FILE)

ms.setResponseDataBody(resultFile)

ms.onResponseData()

ms.onLeaveState()

ms.onFinalize()
