import numpy
import arcpy
from arcpy import env  
from arcpy.sa import *
import os
import shutil

class GIS:
    @staticmethod
    def IDW(workspace, shpfile, field, outputfile, cellsizerasterfile = 0.008062025, extent = arcpy.Extent(115.405368077, 39.4318120751, 117.509556603, 41.0603411261)):
        env.workspace = workspace
        arcpy.env.cellSize = cellsizerasterfile
        arcpy.env.extent = extent
        inPointFeatures = shpfile
        zField = field
        arcpy.CheckOutExtension("Spatial")
        outIDW = Idw(inPointFeatures, zField)
        outIDW.save(outputfile)

    @staticmethod
    def ExtractMulti2Pt(workspace, shpfile, fieldcoll):
        arcpy.CheckOutExtension("Spatial")
        env.workspace = workspace
        ExtractMultiValuesToPoints(shpfile, fieldcoll, "NONE")

    @staticmethod
    def ConvertCSV2Pts(inputfile, projfile, targetshp):
        # "E:\\DemoData\\GeoModeling\\Air\\Data\\DailyData\\D20140101\\Weather_2014_1_1.csv"
        input_file = open(inputfile,"r")
        # "E:\\DemoData\\GeoModeling\\Air\\Data\\Beijing\\sites.prj"
        spatialRef1 = arcpy.SpatialReference(projfile)
        mark = True
        PGList = []
        # elev = []
        tmpmax = []
        tmpmean = []
        tmpmin = []
        pcp = []
        windmean = []
        windmax = []
        sd = []
        tmp0mean = []
        tmp0max = []
        tmp0min = []
        rhmin = []
        rhmean = []
        prsmean = []
        prsmin = []
        prsmax = []
        evamin = []
        evamax = []
        solar = []
        for s in input_file:
            if mark:
                mark = False
            else:
                values = s.split(",")
                point = arcpy.Point(float(values[2].replace('"','')), float(values[3].replace('"','')))
                pointGeometry = arcpy.PointGeometry(point,spatialRef1)
                PGList.append(pointGeometry)
                # elev.append(float(values[3].replace('"','')))
                tmpmax.append(float(values[4].replace('"','')))
                tmpmean.append(float(values[5].replace('"','')))
                tmpmin.append(float(values[6].replace('"','')))
                pcp.append(float(values[7].replace('"','')))
                windmean.append(float(values[8].replace('"','')))
                windmax.append(float(values[9].replace('"','')))
                sd.append(float(values[10].replace('"','')))
                tmp0mean.append(float(values[11].replace('"','')))
                tmp0max.append(float(values[12].replace('"','')))
                tmp0min.append(float(values[13].replace('"','')))
                rhmin.append(float(values[14].replace('"','')))
                rhmean.append(float(values[15].replace('"','')))
                prsmean.append(float(values[16].replace('"','')))
                prsmin.append(float(values[17].replace('"','')))
                prsmax.append(float(values[18].replace('"','')))
                evamin.append(float(values[19].replace('"','')))
                evamax.append(float(values[20].replace('"','')))
                # solar.append(float(values[21].replace('"','')))
        input_file.close()
        # "E:\\DemoData\\GeoModeling\\Air\\Data\\DailyData\\D20140101\\Weather_2014_1_1.shp", "elev", "FLOAT"
        arcpy.CopyFeatures_management(PGList, targetshp)
        arcpy.AddField_management(targetshp, "tmpmax", "FLOAT", 6, 3)
        arcpy.AddField_management(targetshp, "tmpmean", "FLOAT", 6, 3)
        arcpy.AddField_management(targetshp, "tmpmin", "FLOAT", 6, 3)
        arcpy.AddField_management(targetshp, "pcp", "FLOAT", 6, 6)
        arcpy.AddField_management(targetshp, "windmean", "FLOAT", 6, 6)
        arcpy.AddField_management(targetshp, "windmax", "FLOAT", 6, 6)
        arcpy.AddField_management(targetshp, "sd", "FLOAT", 6, 6)
        arcpy.AddField_management(targetshp, "tmp0mean", "FLOAT", 6, 6)
        arcpy.AddField_management(targetshp, "tmp0max", "FLOAT", 6, 6)
        arcpy.AddField_management(targetshp, "tmp0min", "FLOAT", 6, 6)
        arcpy.AddField_management(targetshp, "rhmin", "FLOAT", 6, 6)
        arcpy.AddField_management(targetshp, "rhmean", "FLOAT", 6, 6)
        arcpy.AddField_management(targetshp, "prsmean", "FLOAT", 6, 6)
        arcpy.AddField_management(targetshp, "prsmin", "FLOAT", 6, 6)
        arcpy.AddField_management(targetshp, "prsmax", "FLOAT", 6, 6)
        arcpy.AddField_management(targetshp, "evamin", "FLOAT", 6, 6)
        arcpy.AddField_management(targetshp, "evamax", "FLOAT", 6, 6)

        cur = arcpy.UpdateCursor(targetshp)
        i = 0
        for row in cur:
            # row.elev = elev[i]
            row.tmpmax = tmpmax[i]
            row.tmpmean = tmpmean[i]
            row.tmpmin = tmpmin[i]
            row.pcp = pcp[i]
            row.windmean = windmean[i]
            row.windmax = windmax[i]
            row.sd = sd[i]
            row.tmp0mean = tmp0mean[i]
            row.tmp0max = tmp0max[i]
            row.tmp0min = tmp0min[i]
            row.rhmin = rhmin[i]
            row.rhmean = rhmean[i]
            row.prsmean = prsmean[i]
            row.prsmin = prsmin[i]
            row.prsmax = prsmax[i]
            row.evamin = evamin[i]
            row.evamax = evamax[i]

            i = i + 1
            cur.updateRow(row)
        del cur
        del row
        

class FileOpera:
    @staticmethod
    def CopyFile(srcfile, dstfile):
        if not os.path.isfile(srcfile):
            print "%s not exist!"%(srcfile)
        else:
            fpath,fname=os.path.split(dstfile)
            if not os.path.exists(fpath):
                os.makedirs(fpath)
            shutil.copyfile(srcfile,dstfile) 
            print "copy %s -> %s"%( srcfile,dstfile)
    
    @staticmethod
    def CopyShpfile(src, dst):
        dir_source_dbf = src + '.dbf'
        dir_source_shp = src + '.shp'
        dir_source_prj = src + '.prj'
        dir_source_sbn = src + '.sbn'
        dir_source_sbx = src + '.sbx'
        dir_source_shx = src + '.shx'

        dir_target_dbf = dst + '.dbf'
        dir_target_shp = dst + '.shp'
        dir_target_prj = dst + '.prj'
        dir_target_sbn = dst + '.sbn'
        dir_target_sbx = dst + '.sbx'
        dir_target_shx = dst + '.shx'

        FileOpera.CopyFile(dir_source_dbf, dir_target_dbf)
        FileOpera.CopyFile(dir_source_shp, dir_target_shp)
        FileOpera.CopyFile(dir_source_prj, dir_target_prj)
        FileOpera.CopyFile(dir_source_sbn, dir_target_sbn)
        FileOpera.CopyFile(dir_source_sbx, dir_target_sbx)
        FileOpera.CopyFile(dir_source_shx, dir_target_shx)